# This file is autogenerated by the command `make fix-copies`, do not edit.
from ..file_utils import requires_pytorch


class PyTorchBenchmark:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class PyTorchBenchmarkArguments:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class DataCollator:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class DataCollatorForLanguageModeling:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class DataCollatorForNextSentencePrediction:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class DataCollatorForPermutationLanguageModeling:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class DataCollatorForSOP:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class DataCollatorForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class DataCollatorForWholeWordMask:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class DataCollatorWithPadding:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


def default_data_collator(*args, **kwargs):
    requires_pytorch(default_data_collator)


class GlueDataset:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class GlueDataTrainingArguments:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class LineByLineTextDataset:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class LineByLineWithRefDataset:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class LineByLineWithSOPTextDataset:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class SquadDataset:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class SquadDataTrainingArguments:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class TextDataset:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class TextDatasetForNextSentencePrediction:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class BeamScorer:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class BeamSearchScorer:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class LogitsProcessor:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class LogitsProcessorList:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class LogitsWarper:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class MinLengthLogitsProcessor:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class NoBadWordsLogitsProcessor:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class NoRepeatNGramLogitsProcessor:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class RepetitionPenaltyLogitsProcessor:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class TemperatureLogitsWarper:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class TopKLogitsWarper:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class TopPLogitsWarper:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


def top_k_top_p_filtering(*args, **kwargs):
    requires_pytorch(top_k_top_p_filtering)


ALBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class AlbertForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AlbertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AlbertForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class AlbertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AlbertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AlbertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AlbertModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AlbertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


def load_tf_weights_in_albert(*args, **kwargs):
    requires_pytorch(load_tf_weights_in_albert)


MODEL_FOR_CAUSAL_LM_MAPPING = None


MODEL_FOR_MASKED_LM_MAPPING = None


MODEL_FOR_MULTIPLE_CHOICE_MAPPING = None


MODEL_FOR_PRETRAINING_MAPPING = None


MODEL_FOR_QUESTION_ANSWERING_MAPPING = None


MODEL_FOR_SEQ_TO_SEQ_CAUSAL_LM_MAPPING = None


MODEL_FOR_SEQUENCE_CLASSIFICATION_MAPPING = None


MODEL_FOR_TOKEN_CLASSIFICATION_MAPPING = None


MODEL_MAPPING = None


MODEL_WITH_LM_HEAD_MAPPING = None


class AutoModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AutoModelForCausalLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AutoModelForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AutoModelForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AutoModelForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AutoModelForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AutoModelForSeq2SeqLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AutoModelForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AutoModelForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class AutoModelWithLMHead:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


BART_PRETRAINED_MODEL_ARCHIVE_LIST = None


class BartForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class BartForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class BartForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class BartModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class PretrainedBartModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


BERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class BertForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class BertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class BertForNextSentencePrediction:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class BertForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class BertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class BertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class BertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class BertLayer:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class BertLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class BertModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class BertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


def load_tf_weights_in_bert(*args, **kwargs):
    requires_pytorch(load_tf_weights_in_bert)


class BertGenerationDecoder:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class BertGenerationEncoder:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


def load_tf_weights_in_bert_generation(*args, **kwargs):
    requires_pytorch(load_tf_weights_in_bert_generation)


BLENDERBOT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class BlenderbotForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


CAMEMBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class CamembertForCausalLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class CamembertForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class CamembertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class CamembertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class CamembertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class CamembertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class CamembertModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


CTRL_PRETRAINED_MODEL_ARCHIVE_LIST = None


class CTRLLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class CTRLModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class CTRLPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


DEBERTA_PRETRAINED_MODEL_ARCHIVE_LIST = None


class DebertaForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class DebertaModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class DebertaPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


DISTILBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class DistilBertForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class DistilBertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class DistilBertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class DistilBertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class DistilBertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class DistilBertModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class DistilBertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class DPRContextEncoder:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class DPRPretrainedContextEncoder:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class DPRPretrainedQuestionEncoder:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class DPRPretrainedReader:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class DPRQuestionEncoder:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class DPRReader:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


ELECTRA_PRETRAINED_MODEL_ARCHIVE_LIST = None


class ElectraForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class ElectraForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class ElectraForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class ElectraForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class ElectraForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class ElectraForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class ElectraModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class ElectraPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


def load_tf_weights_in_electra(*args, **kwargs):
    requires_pytorch(load_tf_weights_in_electra)


class EncoderDecoderModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


FLAUBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class FlaubertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FlaubertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FlaubertForQuestionAnsweringSimple:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FlaubertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FlaubertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FlaubertModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FlaubertWithLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FSMTForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FSMTModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class PretrainedFSMTModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


FUNNEL_PRETRAINED_MODEL_ARCHIVE_LIST = None


class FunnelBaseModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FunnelForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FunnelForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FunnelForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class FunnelForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FunnelForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FunnelForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class FunnelModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


def load_tf_weights_in_funnel(*args, **kwargs):
    requires_pytorch(load_tf_weights_in_funnel)


GPT2_PRETRAINED_MODEL_ARCHIVE_LIST = None


class GPT2DoubleHeadsModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class GPT2ForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class GPT2LMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class GPT2Model:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class GPT2PreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


def load_tf_weights_in_gpt2(*args, **kwargs):
    requires_pytorch(load_tf_weights_in_gpt2)


LAYOUTLM_PRETRAINED_MODEL_ARCHIVE_LIST = None


class LayoutLMForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class LayoutLMForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class LayoutLMModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


LONGFORMER_PRETRAINED_MODEL_ARCHIVE_LIST = None


class LongformerForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class LongformerForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class LongformerForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class LongformerForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class LongformerForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class LongformerModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class LongformerSelfAttention:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class LxmertEncoder:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class LxmertForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class LxmertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class LxmertModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class LxmertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class LxmertVisualFeatureEncoder:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class LxmertXLayer:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class MarianMTModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class MBartForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class MMBTForClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class MMBTModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class ModalEmbeddings:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


MOBILEBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class MobileBertForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class MobileBertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class MobileBertForNextSentencePrediction:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class MobileBertForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class MobileBertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class MobileBertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class MobileBertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class MobileBertLayer:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class MobileBertModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class MobileBertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


def load_tf_weights_in_mobilebert(*args, **kwargs):
    requires_pytorch(load_tf_weights_in_mobilebert)


OPENAI_GPT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class OpenAIGPTDoubleHeadsModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class OpenAIGPTForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class OpenAIGPTLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class OpenAIGPTModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class OpenAIGPTPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


def load_tf_weights_in_openai_gpt(*args, **kwargs):
    requires_pytorch(load_tf_weights_in_openai_gpt)


class PegasusForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


PROPHETNET_PRETRAINED_MODEL_ARCHIVE_LIST = None


class ProphetNetDecoder:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class ProphetNetEncoder:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class ProphetNetForCausalLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class ProphetNetForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class ProphetNetModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class ProphetNetPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class RagModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class RagSequenceForGeneration:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class RagTokenForGeneration:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


REFORMER_PRETRAINED_MODEL_ARCHIVE_LIST = None


class ReformerAttention:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class ReformerForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class ReformerForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class ReformerForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class ReformerLayer:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class ReformerModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class ReformerModelWithLMHead:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


RETRIBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class RetriBertModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class RetriBertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST = None


class RobertaForCausalLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class RobertaForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class RobertaForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class RobertaForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class RobertaForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class RobertaForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class RobertaModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


SQUEEZEBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class SqueezeBertForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class SqueezeBertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class SqueezeBertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class SqueezeBertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class SqueezeBertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class SqueezeBertModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class SqueezeBertModule:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class SqueezeBertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


T5_PRETRAINED_MODEL_ARCHIVE_LIST = None


class T5ForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class T5Model:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class T5PreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


def load_tf_weights_in_t5(*args, **kwargs):
    requires_pytorch(load_tf_weights_in_t5)


TRANSFO_XL_PRETRAINED_MODEL_ARCHIVE_LIST = None


class AdaptiveEmbedding:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class TransfoXLLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class TransfoXLModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class TransfoXLPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


def load_tf_weights_in_transfo_xl(*args, **kwargs):
    requires_pytorch(load_tf_weights_in_transfo_xl)


class Conv1D:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class PreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


def apply_chunking_to_forward(*args, **kwargs):
    requires_pytorch(apply_chunking_to_forward)


def prune_layer(*args, **kwargs):
    requires_pytorch(prune_layer)


XLM_PRETRAINED_MODEL_ARCHIVE_LIST = None


class XLMForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLMForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLMForQuestionAnsweringSimple:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLMForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLMForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLMModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLMPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLMWithLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


XLM_PROPHETNET_PRETRAINED_MODEL_ARCHIVE_LIST = None


class XLMProphetNetDecoder:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class XLMProphetNetEncoder:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class XLMProphetNetForCausalLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class XLMProphetNetForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLMProphetNetModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


XLM_ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST = None


class XLMRobertaForCausalLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class XLMRobertaForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLMRobertaForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLMRobertaForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLMRobertaForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLMRobertaForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLMRobertaModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


XLNET_PRETRAINED_MODEL_ARCHIVE_LIST = None


class XLNetForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLNetForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLNetForQuestionAnsweringSimple:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLNetForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLNetForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLNetLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLNetModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


class XLNetPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_pytorch(self)


def load_tf_weights_in_xlnet(*args, **kwargs):
    requires_pytorch(load_tf_weights_in_xlnet)


class Adafactor:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


class AdamW:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


def get_constant_schedule(*args, **kwargs):
    requires_pytorch(get_constant_schedule)


def get_constant_schedule_with_warmup(*args, **kwargs):
    requires_pytorch(get_constant_schedule_with_warmup)


def get_cosine_schedule_with_warmup(*args, **kwargs):
    requires_pytorch(get_cosine_schedule_with_warmup)


def get_cosine_with_hard_restarts_schedule_with_warmup(*args, **kwargs):
    requires_pytorch(get_cosine_with_hard_restarts_schedule_with_warmup)


def get_linear_schedule_with_warmup(*args, **kwargs):
    requires_pytorch(get_linear_schedule_with_warmup)


def get_polynomial_decay_schedule_with_warmup(*args, **kwargs):
    requires_pytorch(get_polynomial_decay_schedule_with_warmup)


class Trainer:
    def __init__(self, *args, **kwargs):
        requires_pytorch(self)


def torch_distributed_zero_first(*args, **kwargs):
    requires_pytorch(torch_distributed_zero_first)



#!/usr/bin/python
import argparse
import json
from tatqa_metric import *
from typing import Any, Dict, Tuple


def evaluate_json(golden_answers: Dict[str, Any], predicted_answers: Dict[str, Any]) -> Tuple[float, float]:

    em_and_f1 = TaTQAEmAndF1()
    for qas in golden_answers:
        for qa in qas["questions"]:
            query_id = qa["uid"]
            pred_answer, pred_scale = None, None
            if query_id in predicted_answers:
                pred_answer, pred_scale = predicted_answers[query_id]
            em_and_f1(ground_truth=qa, prediction=pred_answer, pred_scale=pred_scale)

    global_em, global_f1, global_scale, global_op = em_and_f1.get_overall_metric()
    print("----")
    print("Exact-match accuracy {0:.2f}".format(global_em * 100))
    print("F1 score {0:.2f}".format(global_f1 * 100))
    print("Scale score {0:.2f}".format(global_scale * 100))
    print("{0:.2f}   &   {1:.2f}".format(global_em * 100, global_f1 * 100))
    print("----")

    detail_raw = em_and_f1.get_raw_pivot_table()
    print("---- raw detail ---")
    print(detail_raw)
    detail_em, detail_f1 = em_and_f1.get_detail_metric()
    print("---- em detail ---")
    print(detail_em)
    print("---- f1 detail ---")
    print(detail_f1)


def evaluate_prediction_file(gold_path: str,
                             pred_path: str):

    golden_answers = json.load(open(gold_path, encoding='utf-8'))
    predicted_answers = json.load(open(pred_path, encoding='utf-8'))
    evaluate_json(golden_answers, predicted_answers)


if __name__ == "__main__":
    # pylint: disable=invalid-name
    parser = argparse.ArgumentParser(description='evaluation on TAT-QA dataset')
    parser.add_argument("--gold_path",
                        type=str,
                        required=True,
                        default="tatqa_dataset_test_gold.json",
                        help='The path of the gold file')
    parser.add_argument("--pred_path",
                        type=str,
                        required=True,
                        default="sample_predictions.json",
                        help='The path of the prediction file')

    args = parser.parse_args()
    evaluate_prediction_file(args.gold_path, args.pred_path)



# This file is autogenerated by the command `make fix-copies`, do not edit.
from ..file_utils import requires_flax


class FlaxBertModel:
    def __init__(self, *args, **kwargs):
        requires_flax(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_flax(self)


class FlaxRobertaModel:
    def __init__(self, *args, **kwargs):
        requires_flax(self)

    @classmethod
    def from_pretrained(self, *args, **kwargs):
        requires_flax(self)



import torch
from argparse import ArgumentParser


def add_data_args(parser: ArgumentParser):
    parser.add_argument("--gpu_num", default=torch.cuda.device_count(), type=int, help="training gpu num.")
    parser.add_argument("--data_dir", default="", type=str, required=True, help="data dir.")
    parser.add_argument("--save_dir", default="", type=str, required=True, help="save dir.")
    parser.add_argument("--log_file", default="train.log", type=str, help="train log file.")

def add_train_args(parser: ArgumentParser):
    parser.add_argument('--gradient_accumulation_steps', type=int, default=1,
                        help="Number of updates steps to accumulate before performing a backward/update pass.")
    parser.add_argument("--log_per_updates", default=20, type=int, help="log pre update size.")
    parser.add_argument("--max_epoch", default=5, type=int, help="max epoch.")
    parser.add_argument("--weight_decay", default=0.01, type=float, help="weight decay.")
    parser.add_argument("--learning_rate", default=5e-5, type=float, help="learning rate.")
    parser.add_argument("--grad_clipping", default=1.0, type=float, help="gradient clip.")
    parser.add_argument('--warmup', type=float, default=0.1,
                        help="Proportion of training to perform linear learning rate warmup for. "
                             "E.g., 0.1 = 10%% of training.")
    parser.add_argument("--warmup_schedule", default="warmup_linear", type=str, help="warmup schedule.")
    parser.add_argument("--optimizer", default="adam", type=str, help="train optimizer.")
    parser.add_argument('--seed', type=int, default=2018, help='random seed for data shuffling, embedding init, etc.')
    parser.add_argument('--pre_path', type=str, default=None, help="Load from pre trained.")
    parser.add_argument("--dropout", default=0.2, type=float, help="dropout.")
    parser.add_argument('--batch_size', type=int, default=32, help="batch size.")
    parser.add_argument('--eval_batch_size', type=int, default=32, help="eval batch size.")
    parser.add_argument("--eps", default=1e-8, type=float, help="ema gamma.")
    parser.add_argument("--model_path", default='./pctqa1_g', type=str)

def add_bert_args(parser: ArgumentParser):
    parser.add_argument("--bert_learning_rate", type=float, help="bert learning rate.")
    parser.add_argument("--bert_weight_decay", type=float, help="bert weight decay.")
    parser.add_argument("--roberta_model", type=str, help="robert model path.", default="dataset_reghnt/roberta.large")
    parser.add_argument("--bert_model", type=str, help="bert model path.", default="dataset_reghnt/bert.large")

def add_inference_args(parser: ArgumentParser):
    parser.add_argument("--pre_path", type=str, help="Prepath")
    parser.add_argument("--data_mode", type=str, help="inference data mode")
    parser.add_argument("--inf_path", type=str, help="inference data path.")
    parser.add_argument("--dump_path", type=str, help="inference data path.")
    parser.add_argument('--eval_batch_size', type=int, default=32, help="eval batch size.")



import os
import json
import time
import torch
from typing import Dict, Optional
from datetime import timedelta

from transformers import (
    Seq2SeqTrainer,
    TrainerCallback,
    TrainerControl,
    TrainerState,
    TrainingArguments
)

from transformers.trainer import TRAINING_ARGS_NAME
from transformers.modeling_utils import unwrap_model

from .utils import (
    get_logger,
    FinetuningArguments,
    FINETUNING_ARGS_NAME,
    VALUE_HEAD_FILE_NAME
)

from .model import (
    get_state_dict,
    load_trainable_params,
    load_valuehead_params
)


logger = get_logger(__name__)


class LogCallback(TrainerCallback):
    r"""
    TrainerCallback includes the state function during training, for more details refer to the TrainerCallback class.
    The on_log function primarily collects process parameters during training, such as training loss, learning rate,
    and training epochs, as well as progress parameters like the current percentage progress and estimated remaining
    time. Every time a log is triggered, a new record is appended to the file "messages.log" for dynamic visualization
    purposes.
    """

    def __init__(self):
        self.start_time = time.time()

    def on_log(self, args: TrainingArguments, state: TrainerState, control: TrainerControl, **kwargs) -> None:
        r"""
        Event called after logging the last logs.
        """
        if "loss" not in state.log_history[-1]:
            return
        cur_time = time.time()
        cur_steps = state.log_history[-1].get("step")
        elapsed_time = cur_time - self.start_time
        avg_time_per_step = elapsed_time / cur_steps if cur_steps != 0 else 0
        remaining_steps = state.max_steps - cur_steps
        remaining_time = remaining_steps * avg_time_per_step
        log_dict = {
            "current_steps": cur_steps,
            "total_steps": state.max_steps,
            "loss": state.log_history[-1].get("loss", None),
            "reward": state.log_history[-1].get("reward", None),
            "learning_rate": state.log_history[-1].get("learning_rate", None),
            "epoch": state.log_history[-1].get("epoch", None),
            "percentage": round(cur_steps / state.max_steps * 100, 2) if state.max_steps != 0 else 100,
            "elapsed_time": str(timedelta(seconds=int(elapsed_time))),
            "remaining_time": str(timedelta(seconds=int(remaining_time)))
        }
        os.makedirs(args.output_dir, exist_ok=True)
        with open(os.path.join(args.output_dir, "trainer_log.jsonl"), "a") as f:
            f.write(json.dumps(log_dict) + "\n")


class PeftTrainer(Seq2SeqTrainer):
    r"""
    Inherits Seq2SeqTrainer to support parameter-efficient checkpoints.
    """

    def __init__(self, finetuning_args: FinetuningArguments, **kwargs):
        super().__init__(**kwargs)
        self.finetuning_args = finetuning_args
        if self.is_world_process_zero() and os.path.exists(os.path.join(self.args.output_dir, "trainer_log.jsonl")):
            logger.warning("Previous log file in this folder will be deleted.")
            os.remove(os.path.join(self.args.output_dir, "trainer_log.jsonl"))

    def _save(self, output_dir: Optional[str] = None, state_dict: Optional[Dict[str, torch.Tensor]] = None) -> None:
        r"""
        Saves trainable parameters as model checkpoint.

        This function will only be executed at the process zero.

        Subclass and override to inject custom behavior. It should not be directly used by external scripts.
        """
        output_dir = output_dir if output_dir is not None else self.args.output_dir
        os.makedirs(output_dir, exist_ok=True)
        logger.info(f"Saving model checkpoint to {output_dir}")
        model = unwrap_model(self.model)

        if hasattr(model, "pretrained_model"): # for models with valuehead (currently using LoRA only)
            backbone_model = getattr(model, "pretrained_model")
            torch.save(get_state_dict(getattr(model, "v_head")), os.path.join(output_dir, VALUE_HEAD_FILE_NAME))
        else:
            backbone_model = model

        if self.finetuning_args.finetuning_type == "lora":
            backbone_model.save_pretrained(output_dir, state_dict=get_state_dict(backbone_model))
        else: # freeze/full tuning
            backbone_model.save_pretrained(
                output_dir,
                state_dict=get_state_dict(backbone_model),
                safe_serialization=self.args.save_safetensors
            )
            if self.tokenizer is not None:
                self.tokenizer.save_pretrained(output_dir)

        with open(os.path.join(output_dir, TRAINING_ARGS_NAME), "w", encoding="utf-8") as f:
            f.write(self.args.to_json_string() + "\n")
        self.finetuning_args.save_to_json(os.path.join(output_dir, FINETUNING_ARGS_NAME))

    def _load_best_model(self):
        r"""
        Loads trainable parameters from model checkpoint.

        Subclass and override to inject custom behavior. It should not be directly used by external scripts.
        """
        logger.info(f"Loading best model from {self.state.best_model_checkpoint} (score: {self.state.best_metric}).")

        model = unwrap_model(self.model)
        backbone_model = getattr(model, "pretrained_model") if hasattr(model, "pretrained_model") else model

        if self.finetuning_args.finetuning_type == "lora":
            backbone_model.load_adapter(self.state.best_model_checkpoint, getattr(backbone_model, "active_adapter"))
            if hasattr(model, "v_head") and load_valuehead_params(model, self.state.best_model_checkpoint):
                model.v_head.load_state_dict({
                    "summary.weight": getattr(model, "reward_head_weight"),
                    "summary.bias": getattr(model, "reward_head_bias")
                })
        else: # freeze/full-tuning
            load_trainable_params(backbone_model, self.state.best_model_checkpoint)



import torch
import torch.nn as nn
from torch.utils.data import Dataset, DataLoader
import numpy as np
from transformers import BartTokenizer, BartForQuestionAnswering, BartForConditionalGeneration
from transformers import RobertaTokenizer, RobertaModel, BertTokenizer, BertModel, BertForQuestionAnswering
import json
from tqdm import tqdm
import random
from pathlib import Path
import logging
import os, sys, math
import torch.nn.functional as F
from transformers import get_linear_schedule_with_warmup, AdamW
from evaluate_script import get_raw_scores
import argparse


##  添加了AdamW和linear warm up
def create_logger(name, silent=False, to_disk=True, log_file=None):
    """Logger wrapper
    """
    # setup logger
    log = logging.getLogger(name)
    log.setLevel(logging.DEBUG)
    log.propagate = False
    formatter = logging.Formatter(fmt='%(asctime)s %(message)s', datefmt='%m/%d/%Y %I:%M:%S')
    if not silent:
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logging.INFO)
        ch.setFormatter(formatter)
        log.addHandler(ch)
    if to_disk:
        log_file = log_file if log_file is not None else strftime("%Y-%m-%d-%H-%M-%S.log", gmtime())
        fh = logging.FileHandler(log_file)
        fh.setLevel(logging.DEBUG)
        fh.setFormatter(formatter)
        log.addHandler(fh)
    return log

def load_data(path):
    with open(path, 'r') as f:
        data = json.load(f)
    return data

def is_whitespace(c):
    if c == " " or c == "\t" or c == "\r" or c == "\n" or ord(c) == 0x202F:
        return True
    return False

def gelu(x):
    return x * 0.5 * (1.0 + torch.erf(x / math.sqrt(2.0)))






class TypeDataset(Dataset):
    def __init__(self, tokenizer, data, is_train, data_use, rerank_link, is_test=0) -> None:
        super(TypeDataset, self).__init__()
        self.tokenizer = tokenizer
        self.ori_data = []
        self.is_train = is_train
        self.is_test = is_test
        self.rerank_link = rerank_link


        if data_use == 0:    
            self.ori_data = data
        elif data_use == 1:  
            for item in data:
                if sum(item['labels']) != 0:
                    self.ori_data.append(item)
        elif data_use == 2:   
            for item in data:
                if sum(item['labels']) == 1:
                   self.ori_data.append(item)

        total_data = []
        for data in tqdm(self.ori_data):
            path = './Data/HybridQA/WikiTables-WithLinks'
            table_id = data['table_id']
            with open('{}/tables_tok/{}.json'.format(path, table_id), 'r') as f:
                table = json.load(f)  
            with open('{}/request_tok/{}.json'.format(path, table_id), 'r') as f:
                requested_document = json.load(f)
            dot_token = self.tokenizer.additional_special_tokens[0]
            dot_token_id = self.tokenizer.convert_tokens_to_ids(dot_token)
            headers = [_[0] for _ in table['header']]
            row_tmp = '{} is {} {}'
            if not is_test:
                answer = data['answer-text']
            type = 0
            if data['type'] == 'comparison':
                type = 1
            if is_train:
                if type == 0:
                    if len(data['row_gold']) > 1:
                        logit = data['row_pre_logit']
                        gold_row = [data['row_gold'][np.argmax(np.array(logit)[data['row_gold']])]]
                    elif len(data['row_gold']) == 0:
                        gold_row = [data['row_pre']]
                    else:
                        gold_row = data['row_gold']
                else:
                    if len(data['row_gold']) == 0:
                        gold_row = np.argsort(-np.array(data['row_pre_logit']))[:3].tolist()
                    elif len(data['row_gold']) > 1:
                        logit = data['row_pre_logit']
                        gold_row = [data['row_gold'][np.argmax(np.array(logit)[data['row_gold']])]]
                        predict_rows = np.argsort(-np.array(data['row_pre_logit']))[:3].tolist()
                        if gold_row[0] in predict_rows:
                            gold_row = predict_rows
                        else:
                            gold_row = [gold_row[0]] + predict_rows[:2]
                    elif len(data['row_gold']) == 1:
                        gold_row = data['row_gold']
                        predict_rows = np.argsort(-np.array(data['row_pre_logit']))[:3].tolist()
                        if gold_row[0] in predict_rows:
                            gold_row = predict_rows
                        else:
                            gold_row = [gold_row[0]] + predict_rows[:2]
            else:
                if type == 0:
                    gold_row = [data['row_pre']]
                else:
                    gold_row = np.argsort(-np.array(data['row_pre_logit']))[:3].tolist()

            if type == 0:
                row = table['data'][gold_row[0]]
                question_ids = self.tokenizer.encode(data['question'])
                input_ids = []
                row_ids = []
                links = []
                for j, cell in enumerate(row):
                    if cell[0] != '':
                        cell_desc = row_tmp.format(headers[j], cell[0], dot_token)
                        cell_toks = self.tokenizer.tokenize(cell_desc)
                        cell_ids = self.tokenizer.convert_tokens_to_ids(cell_toks)
                        row_ids += cell_ids
                    if cell[1] != []:
                        links += cell[1]
                links_ids = []
                if rerank_link:
                    links = self.generate_new_links(links, data)
                for link in links:
                    passage_toks = self.tokenizer.tokenize(requested_document[link])
                    passage_ids = self.tokenizer.convert_tokens_to_ids(passage_toks)
                    links_ids += passage_ids + [dot_token_id]
                input_ids = question_ids + row_ids + links_ids + [self.tokenizer.sep_token_id]
            else:
                row_p_dict = data['row_p_dict']
                question_ids = self.tokenizer.encode(data['question'])
                input_ids = question_ids
                for row_id in gold_row:
                    row_ids = []
                    row_links = row_p_dict[row_id]
                    row = table['data'][row_id]
                    links_rank = data['links_rank'][:len(data['links_rank'])//3]
                    if is_train:
                        gold_link = data['gold_link']
                    for j, cell in enumerate(row):
                        if cell[0] != '':
                            cell_desc = row_tmp.format(headers[j], cell[0], dot_token)
                            cell_toks = self.tokenizer.tokenize(cell_desc)
                            cell_ids = self.tokenizer.convert_tokens_to_ids(cell_toks)
                            row_ids += cell_ids
                    if is_train:
                        new_links = list(set(row_links) & set(gold_link) | set(row_links) & set(links_rank))
                    else:
                        new_links = list(set(row_links) & set(links_rank))
                    links = [data['links'][item] for item in new_links]
                    links_ids = []
                    if rerank_link:
                        links = self.generate_new_links(links, data)
                    for link in links:
                        passage_toks = self.tokenizer.tokenize(requested_document[link])
                        passage_ids = self.tokenizer.convert_tokens_to_ids(passage_toks)
                        links_ids += passage_ids + [dot_token_id]
                    input_ids += row_ids + links_ids + [self.tokenizer.sep_token_id]
            data['input_ids'] = input_ids
            if not is_test:
                answer_ids = self.tokenizer.convert_tokens_to_ids(self.tokenizer.tokenize(answer))
                data['answer_ids'] = [self.tokenizer.eos_token_id] + answer_ids + [self.tokenizer.eos_token_id]
            total_data.append(data)
        self.data = total_data


    def string_tokenizer(self, tokenizer, input_string):
        str_tokens = input_string.split(' ')
        str_input_ids = []
        str_tokens_lens = []
        for i, token in enumerate(str_tokens):
            str_input_ids_i = tokenizer.convert_tokens_to_ids(tokenizer.tokenize(token))
            str_input_ids.extend(str_input_ids_i)
            str_tokens_lens.extend([len(str_input_ids_i)])
        return str_tokens, str_input_ids, str_tokens_lens

    def generate_new_links(self, links, data):
        if self.is_train:
            answer_link = [item[2] for item in data['answer-node']]
            new_links, other_links = [], []
            for link in links:
                if link in answer_link:
                    new_links.append(link)
                else:
                    other_links.append(link)
            new_links += other_links
            return new_links
        else:
            aa = 1
            links_rank = data['links_rank']
            total_links = data['links']
            row_link_id = [total_links.index(item) for item in links]
            row_link_id_rank = [links_rank.index(item) for item in row_link_id]
            final_rank = np.argsort(row_link_id_rank).tolist()
            new_links = []
            for item in final_rank:
                new_links.append(links[item])
            return new_links

    def __len__(self):
        return len(self.data)
    def __getitem__(self, index):
        data = self.data[index]
        input_ids = data['input_ids']
        if not self.is_test:
            answer_ids = data['answer_ids']
            return input_ids, answer_ids, data
        else:
            return input_ids, [0], data
            
                    
def collate(data, tokenizer, bert_max_length, is_test=0):
    bs = len(data)
    max_input_length = 0
    input_ids = []
    answer_ids = []
    metadata = []
    max_input_length = max([len(item[0]) for item in data])
    max_answer_length = max([len(item[1]) for item in data])
    if max_input_length > bert_max_length:
        max_input_length = bert_max_length
    for i in range(bs):
        if len(data[i][0]) > max_input_length:
            input_id = data[i][0][:max_input_length]
        else:
            input_id = data[i][0] + (max_input_length - len(data[i][0])) * [tokenizer.pad_token_id]
        input_ids.append(input_id)

        if not is_test:
            if len(data[i][1]) > max_answer_length:
                answer_id = data[i][1][:max_answer_length]
            else:
                answer_id = data[i][1] + (max_answer_length - len(data[i][1])) * [tokenizer.pad_token_id]
            answer_ids.append(answer_id)
        metadata.append(data[i][2])
    input_ids = torch.tensor(input_ids)
    input_mask = torch.where(input_ids==tokenizer.pad_token_id, 0, 1)
    if not is_test:
        answer_ids = torch.tensor(answer_ids)
        answer_mask = torch.where(answer_ids==tokenizer.pad_token_id, 0, 1)
        return {"input_ids": input_ids.cuda(), "input_mask":input_mask.cuda(), "answer_ids":answer_ids.cuda(), "answer_mask":answer_mask.cuda(), "metadata":metadata}
    else:
        return {"input_ids": input_ids.cuda(), "input_mask":input_mask.cuda(), "metadata":metadata}


def train(epoch, tokenizer, model, loader, optimizer, scheduler, logger):
    model.train()
    averge_step = len(loader) // 5
    loss_sum, step = 0, 0
    for i, data in enumerate(tqdm(loader)):

        labels = data['answer_ids'][:, 1:].clone()
        labels[labels==tokenizer.pad_token_id] = -100
        outputs = model(input_ids=data['input_ids'], 
                        attention_mask=data['input_mask'], 
                        decoder_input_ids=data['answer_ids'][:, :-1],
                        labels=labels)
        loss = outputs[0]
        optimizer.zero_grad()
        loss.backward()
        torch.nn.utils.clip_grad_norm_(model.parameters(), 0.1)
        optimizer.step()
        scheduler.step()
        loss_sum += loss
        step += 1
        if i % averge_step == 0:
            logger.info("Training Loss [{0:.5f}]".format(loss_sum/step))
            loss_sum, step = 0, 0
def eval(tokenizer, model, loader, logger):
    model.eval()
    total, acc = 0, 0
    outputs = []
    with torch.no_grad():
        for i, data in enumerate(tqdm(loader)):
            generated_ids = model.generate(
                input_ids = data['input_ids'],
                attention_mask=data['input_mask'],
                max_length = 20,
                num_beams = 3,
                early_stopping=True
            )
            metadata = data['metadata']
            for i, item in enumerate(generated_ids):
                total += 1
                output = torch.masked_select(item, item.ge(3))
                pred_answer = tokenizer.decode(output)
                answer = metadata[i]['answer-text']
                question_id = metadata[i]['question_id']

                if pred_answer == answer:
                    acc += 1
                outputs.append({"question_id":question_id, "pred":pred_answer})
    print(f"outputs num: {len(outputs)}")
    return acc / total, outputs

def test_eval(tokenizer, model, loader, logger):
    model.eval()
    outputs = []
    with torch.no_grad():
        for i, data in enumerate(tqdm(loader)):
            generated_ids = model.generate(
                input_ids = data['input_ids'],
                attention_mask=data['input_mask'],
                max_length = 20,
                num_beams = 3,
                early_stopping=True
            )
            metadata = data['metadata']
            for i, item in enumerate(generated_ids):
                output = torch.masked_select(item, item.ge(3))
                pred_answer = tokenizer.decode(output)
                question_id = metadata[i]['question_id']
                outputs.append({"question_id":question_id, "pred":pred_answer})
    print(f"outputs num: {len(outputs)}")
    return outputs

def read_config(path):
    with open(path, 'r') as file:
        config = json.load(file)
    return config

def main():
    device = torch.device("cuda")
    parser = argparse.ArgumentParser()
    parser.add_argument('--ptm_type', type=str, default='bart-large', help='Pre-trained model to use')
    parser.add_argument('--train_data_path', type=str, default='./Data/HybridQA/train.row.json', help='Path to train data')
    parser.add_argument('--dev_data_path', type=str, default='./Data/HybridQA/dev.row.json', help='Path to dev data')
    parser.add_argument('--predict_save_path', type=str, default='./Data/HybridQA/dev_answers.json', help='Path to save predictions')
    parser.add_argument('--batch_size', type=int, default=4, help='Batch size for training')
    parser.add_argument('--epoch_nums', type=int, default=10, help='Number of epochs for training')
    parser.add_argument('--learning_rate', type=float, default=1e-5, help='Learning rate for training')
    parser.add_argument('--adam_epsilon', type=float, default=1e-8, help='Epsilon for Adam optimizer')
    parser.add_argument('--warmup_steps', type=int, default=0, help='Number of warmup steps for learning rate scheduler')
    parser.add_argument('--is_train', type=int, default=1, help='Whether to train the model')
    parser.add_argument('--is_test', type=int, default=0, help='Whether to test the model')
    parser.add_argument('--seed', type=int, default=2001, help='Random seed for reproducibility')
    parser.add_argument('--output_dir', type=str, default='./generation_best', help='Output directory for saving model and logs')
    parser.add_argument('--load_dir', type=str, default='./generation_best', help='Directory for loading model')
    parser.add_argument('--bert_max_length', type=int, default=1024, help='Maximum length of input sequence for BERT')
    parser.add_argument('--config_file', type=str, default='config_read.json')

    args = parser.parse_args()
    
    training_config = read_config(args.config_file)

    train_data_path = args.train_data_path
    dev_data_path = args.dev_data_path
    predict_save_path = args.predict_save_path

    warmup_steps = args.warmup_steps
    is_train = args.is_train
    is_test = args.is_test
    seed = args.seed
    output_dir = args.output_dir
    load_dir = args.load_dir
    bert_max_length = args.bert_max_length
    
    batch_size = training_config['batch_size']
    epoch_nums = training_config['epoch_nums']
    bert_max_length = training_config['bert_max_length']
    learning_rate = training_config['learning_rate']
    adam_epsilon = training_config['adam_epsilon']



    log_file = 'log.txt'
    ckpt_file = 'ckpt.pt'
    load_ckpt_file = 'ckpt.pt'
    n_gpu = torch.cuda.device_count()


    with open('Data/HybridQA/dev_reference.json', 'r') as f:
        reference = json.load(f)

    Path(output_dir).mkdir(parents=True, exist_ok=True)
    Path(load_dir).mkdir(parents=True, exist_ok=True)
    logger = create_logger("Training", log_file=os.path.join(output_dir, log_file))

    random.seed(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    if n_gpu > 0:
        torch.cuda.manual_seed_all(seed)
        
    notice = f'is_train={is_train}, is_test={is_test}, lr={learning_rate}, epoch_num={epoch_nums}, output_dir={output_dir}, load_dir={load_dir}, bert_max_length={bert_max_length}'
    logger.info(notice)
        
    
    logger.info(f"loading data......from {train_data_path} and {dev_data_path}")
    train_data, dev_data = load_data(train_data_path), load_data(dev_data_path)
    logger.info(f"train data: {len(train_data)}, dev data: {len(dev_data)}")
    

    if ptm_type == 'bert-base':
        ptm_path = './PTM/bert-base-uncased'
        logger.info(f"loading PTM model......from {ptm_path}")
        tokenizer = BertTokenizer.from_pretrained(ptm_path)
        bert_model = BertModel.from_pretrained(ptm_path)
        special_tokens_dict = {'additional_special_tokens': ['[DOT]']}
        tokenizer.add_special_tokens(special_tokens_dict)
        bert_model.resize_token_embeddings(len(tokenizer))
    elif ptm_type == 'bart-large':
        ptm_path = './PTM/bart-large'
        logger.info(f"loading PTM model......from {ptm_path}")
        tokenizer = BartTokenizer.from_pretrained(ptm_path)
        model = BartForConditionalGeneration.from_pretrained(ptm_path)
        special_tokens_dict = {'additional_special_tokens': ['<dot>']}
        tokenizer.add_special_tokens(special_tokens_dict)
        model.resize_token_embeddings(len(tokenizer))


    if is_train:
        train_dataset = TypeDataset(tokenizer, train_data, is_train=1, data_use=0, rerank_link=1)
    dev_dataset = TypeDataset(tokenizer, dev_data, is_train=0, data_use=0, rerank_link=1, is_test=is_test)


    if is_train:
        train_loader = DataLoader(train_dataset, batch_size=batch_size, collate_fn=lambda x: collate(x, tokenizer, bert_max_length))
    dev_loader = DataLoader(dev_dataset, batch_size=batch_size, collate_fn=lambda x: collate(x, tokenizer, bert_max_length, is_test))
    if is_train:
        logger.info(f"train dataset: {len(train_dataset)}")
    logger.info(f"dev dataset: {len(dev_dataset)}")

    model.to(device)



    best_acc = -1
    if is_train:
        if n_gpu > 1:
            model = torch.nn.DataParallel(model)
        optimizer = AdamW(model.parameters(), lr=learning_rate, eps=adam_epsilon)
        t_total = len(train_dataset) // batch_size * epoch_nums
        scheduler = get_linear_schedule_with_warmup(
            optimizer, num_warmup_steps=warmup_steps, num_training_steps=t_total
        )

        for epoch in range(epoch_nums):
            logger.info(f"Training epoch: {epoch}")
            train(epoch, tokenizer, model, train_loader, optimizer, scheduler, logger)
            logger.info("start eval....")
            total_exact, outputs = eval(tokenizer, model, dev_loader, logger)
            scores = get_raw_scores(outputs, reference)
            total_exact = scores['total exact']
            logger.info(f"{scores}")
            logger.info(f"Total Exact: {total_exact}")
            if total_exact > best_acc:
                best_acc = total_exact
                model_to_save = model.module if hasattr(model, "module") else model
                model_save_path = os.path.join(output_dir, ckpt_file)
                logger.info(f"saving model...to {model_save_path}")
                torch.save(model_to_save.state_dict(), model_save_path)
    else:
        model_load_path = os.path.join(load_dir, load_ckpt_file)
        logger.info(f"loading trained parameters from {model_load_path}")
        model.load_state_dict(torch.load(model_load_path))
        logger.info("start eval....")
        if not is_test:
            _, outputs = eval(tokenizer, model, dev_loader, logger)
            scores = get_raw_scores(outputs, reference)
            total_exact = scores['total exact']
            logger.info(f"{scores}")
            logger.info(f"Total Exact: {total_exact}")
            with open(predict_save_path, 'w') as f:
                json.dump(outputs, f, indent=2)
            logger.info(f"answer save to {predict_save_path}")
        else:
            outputs = test_eval(tokenizer, model, dev_loader, logger)
            with open(predict_save_path, 'w') as f:
                json.dump(outputs, f, indent=2)
            logger.info(f"answer save to {predict_save_path}")
    

if __name__ == '__main__':
    main()


import io, requests, zipfile
import os
import json
import argparse
from datetime import datetime
from reg_hnt import options
import torch
import torch.nn as nn
from pprint import pprint
from reg_hnt.data.data_util import OPERATOR_CLASSES_
from reg_hnt.reghnt.util import create_logger, set_environment
from reg_hnt.data.tatqa_batch_gen import TaTQABatchGen
from reg_hnt.data.tatqa_test_batch_gen import TaTQATestBatchGen
from transformers import RobertaModel, BertModel, ElectraModel, AutoModel
from reg_hnt.reghnt.modeling_reghnt import RegHNTModel
from reg_hnt.reghnt.model import RegHNTFineTuningModel, RegHNTPredictModel
from pathlib import Path
from reg_hnt.reghnt.Vocabulary import *
import numpy as np

class NpEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, (np.int_, np.intc, np.intp, np.int8,
            np.int16, np.int32, np.int64, np.uint8,
            np.uint16, np.uint32, np.uint64)):
            return int(obj)
        elif isinstance(obj, (np.float_, np.float16, np.float32, 
            np.float64)):
            return float(obj)
        elif isinstance(obj, (np.ndarray,)): # add this line
            return obj.tolist() # add this line
        return json.JSONEncoder.default(self, obj) 



parser = argparse.ArgumentParser("RegHNT predicting task.")
options.add_data_args(parser)
options.add_bert_args(parser)
parser.add_argument("--eval_batch_size", type=int, default=1)
parser.add_argument("--model_path", type=str, default="checkpoint")
parser.add_argument("--op_mode", type=int, default=0)
parser.add_argument("--ablation_mode", type=int, default=0)
parser.add_argument("--encoder", type=str, default='roberta_large')
parser.add_argument("--test_data_dir", type=str, default="reg_hnt/cache/")
parser.add_argument("--mode", type=str, default='test')
parser.add_argument("--step", type=str, default='op')

args = parser.parse_args()
if args.ablation_mode != 0:
    args.model_path = args.model_path + "_{}_{}".format(args.op_mode, args.ablation_mode)
if args.ablation_mode != 0:
    args.data_dir = args.data_dir + "_{}_{}".format(args.op_mode, args.ablation_mode)

if not os.path.exists(args.save_dir):
    os.mkdir(args.save_dir)

args.cuda = args.gpu_num > 0

logger = create_logger("RegHNT Predictor", log_file=os.path.join(args.save_dir, args.log_file))

pprint(args)
set_environment(args.cuda)

def main():
    if args.mode == 'dev':
        dev_itr = TaTQABatchGen(args, data_mode='dev', encoder=args.encoder)
    elif args.mode == 'test':
        dev_itr = TaTQATestBatchGen(args, data_mode='test', encoder=args.encoder)

    if args.encoder == 'bert':
        bert_model = BertModel.from_pretrained(args.bert_model)
    elif args.encoder == 'roberta_base':
        bert_model = RobertaModel.from_pretrained(args.roberta_model)
    elif args.encoder == 'roberta_large':
        bert_model = RobertaModel.from_pretrained(args.roberta_model)
        
    if args.ablation_mode == 0:
        operators = OPERATOR_CLASSES_

    if args.ablation_mode == 0:
        arithmetic_op_index = [3, 4, 6, 7, 8, 9]

    network = RegHNTModel(
        encoder = bert_model,
        config = bert_model.config,
        bsz = args.eval_batch_size,
        operator_classes = len(OPERATOR_CLASSES),
        scale_classes = len(SCALE),
        operator_criterion = nn.CrossEntropyLoss(),
        scale_criterion = nn.CrossEntropyLoss(),
        arithmetic_op_index = arithmetic_op_index,
        op_mode = args.op_mode,
        ablation_mode = args.ablation_mode,
    )

    network.load_state_dict(torch.load(os.path.join(args.model_path,"checkpoint_best.pt")))
    model = RegHNTPredictModel(args, network)
    
    logger.info("Below are the result on Dev set...")
    model.reset()
    model.avg_reset()

    if args.mode == 'dev':
        pred_json = model.predict(dev_itr)
        model.get_metrics()
        json.dump(pred_json,open(os.path.join(args.save_dir, 'pred_result_on_dev.json'), 'w'),ensure_ascii=False,cls=NpEncoder)
    elif args.mode == 'test':
        pred_json = model.predict2(dev_itr)
        json.dump(pred_json,open(os.path.join(args.save_dir, 'pred_result_on_test.json'), 'w'),ensure_ascii=False,cls=NpEncoder)



if __name__ == "__main__":
    main()


CONST_LIST = ['0', '1', '2', '3', '4', '5', '6', '7', '8']
CONST_LIST2 = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '12','100','1000']
OPERATOR_LIST = ['+', '-', '*', '/']
OPERATOR_CLASSES = {"SPAN": 0, "MULTI_SPAN": 1, "COUNT": 2, "ARITHMETIC": 3}
OPERATOR_CLASSES_R = {0:"SPAN", 1:"MULTI_SPAN", 2:"COUNT" ,3:"ARITHMETIC"}
SCALE = ["", "thousand", "million", "billion", "percent"]

RC_OPERATOR_LIST = ['*', '+']
RC_CONST_LIST = []

Relation_vocabulary = {
    "word2id": {
    "Row-Contain-Cell":0, "Cell-Belong-Row":1,
    "Column-Contain-Cell":2, "Cell-Belong-Column":3,
    "Time-Contain-Cell":4, "Cell-Belong-Time":5,
    "Cell-SameRow-Cell":6, 
    "Q_Word-NextWord-Q_Word":7, "Q_Word-PriorWord-Q_Word":8,####
    "Q_Word-PartialMatch-Row":9, "Row-PartialMatch-Q_Word":10,
    "Q_Word-ExactMatch-Row":11, "Row-ExactMatch-Q_Word":12,
    "Q_Word-Partof-Sentence":13, "Sentence-Contain-Q_Word":14,
    "P_Word-Partof-Sentence":15, "Sentence-Contain-P_Word":16,
    "Sentence-Contain-Row":17, "Row-Partof-Sentence":18,
    "Word-Same-Word": 19, 
    "P_Word-PartialMatch-Row":20,"Row-PartialMatch-P_Word":21, 
    "P_Word-ExactMatch-Row":22,"Row-ExactMatch-P_Word": 23,
    "self-same-self":24
},
    "id2word": {
    0:"Row-Contain-Cell", 1:"Cell-Belong-Row",
    2:"Column-Contain-Cell", 3:"Cell-Belong-Column",
    4:"Time-Contain-Cell", 5:"Cell-Belong-Time",
    6:"Cell-SameRow-Cell", 
    7:"Q_Word-NextWord-Q_Word", 8:"Q_Word-PriorWord-Q_Word",####
    9:"Q_Word-PartialMatch-Row", 10:"Row-PartialMatch-Q_Word",
    11:"Q_Word-ExactMatch-Row", 12:"Row-ExactMatch-Q_Word",
    13:"Q_Word-Partof-Sentence", 14:"Sentence-Contain-Q_Word",
    15:"P_Word-Partof-Sentence", 16:"Sentence-Contain-P_Word",
    17:"Sentence-Contain-Row", 18:"Row-Partof-Sentence",
    19:"Word-Same-Word",
    20:"P_Word-PartialMatch-Row", 21:"Row-PartialMatch-P_Word",
    22:"P_Word-ExactMatch-Row", 23:"Row-ExactMatch-P_Word",
    24:"self-same-self"
    }
}


import os
import pickle
import argparse
from reg_hnt.data.tatqa_dataset import TaTQAReader
from reg_hnt.data.tatqa_dataset_test import TaTQATestReader
from transformers import RobertaTokenizer
from transformers import BertTokenizer

parser = argparse.ArgumentParser()
parser.add_argument("--input_path", type=str, default='./dataset_reghnt')
parser.add_argument("--output_dir", type=str, default="./reg_hnt/cache")
parser.add_argument("--passage_length_limit", type=int, default=512)
parser.add_argument("--question_length_limit", type=int, default=60)
parser.add_argument("--encoder", type=str, default="roberta_large")
parser.add_argument("--mode", type=str, default='train')

args = parser.parse_args()

if args.encoder == 'roberta_base':
    tokenizer = RobertaTokenizer.from_pretrained(args.input_path + "/roberta.base")
    sep = '<s>'
elif args.encoder == 'roberta_large':
    tokenizer = RobertaTokenizer.from_pretrained(args.input_path + "/roberta.large")
    sep = '<s>'
elif args.encoder == 'bert':
    tokenizer = BertTokenizer.from_pretrained(args.input_path + "/bert.large")
    sep = '[SEP]'


if args.mode == 'train':
    data_reader =  TaTQAReader(tokenizer, args.passage_length_limit, args.question_length_limit, mode='train', sep=sep)
    data_mode = {"train"}
elif args.mode == 'dev':
    data_reader =  TaTQAReader(tokenizer, args.passage_length_limit, args.question_length_limit, mode='dev', sep=sep)
    data_mode = {"dev"}
elif args.mode == 'test':
    data_reader = TaTQATestReader(tokenizer, args.passage_length_limit, args.question_length_limit, sep=sep)
    data_mode = {"test"}
elif args.mode == 'test1':
    data_reader = TaTQATestReader(tokenizer, args.passage_length_limit, args.question_length_limit, sep=sep)
    data_mode = {"test1"}

data_format = "tatqa_dataset_{}.json"
print(f'==== NOTE ====: encoder:{args.encoder}, mode:{args.mode}')

for dm in data_mode:
    dpath = os.path.join(args.input_path, data_format.format(dm))
    data = data_reader._read(dpath)
    print(data_reader.skip_count)
    data_reader.skip_count = 0
    print("Save data to {}.".format(os.path.join(args.output_dir, f"reghnt_{args.encoder}_cached_{dm}.pkl")))
    if not os.path.exists(args.output_dir):
        os.makedirs(args.output_dir)
    with open(os.path.join(args.output_dir, f"reghnt_{args.encoder}_cached_{dm}.pkl"), "wb") as f:
        pickle.dump(data, f)



import os
import platform

import re
import logging
import json
from structs.data import Cell, Table, HFTabularDataset
from bs4 import BeautifulSoup

logger = logging.getLogger(__name__)

class MultiHiertt(HFTabularDataset):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.html = []

    def wrap_with_div(self, table_str):
        return f'<div>{table_str}</div>'

    def _load_split(self, split):
        question_file_map = {
            "train": "train.json",
            "dev": "dev.json",
            "test": "test.json"
        }

        file_base_path = 'datasets/MultiHiertt'
        logger.info(f"Loading MultiHiertt - {split}")

        question_file_path = os.path.join(file_base_path, question_file_map[split])

        with open(question_file_path, 'r', encoding='utf-8') as f:
            question_data_list = json.load(f)

        data = []

        for question_data in question_data_list:
            properties_info = {}

            if 'program' in question_data['qa']:
                properties_info['program'] = str(question_data['qa']['program'])
            if 'question_type' in question_data['qa']:
                properties_info['question_type'] = str(question_data['qa']['question_type'])

            question = question_data['qa']['question']
            
            html_data = ""
            for table_html in question_data["tables"]:

                soup = BeautifulSoup(table_html, 'html.parser')

      
                for table in soup.find_all('table'):
                    table['class'] = "table table-sm no-footer table-bordered caption-top main-table"

           
                updated_html_string = soup.prettify()


                html_data += self.wrap_with_div(updated_html_string)
            
            txt_list = question_data['paragraphs']
            
            data.append({
                "question": question,
                "html": html_data,
                "txt": txt_list,
                "properties": properties_info
            })
        self.data[split] = data
        self.dataset_info = {
            "citation": "111",
            "description": "111",
            "version": "111",
            "license": "111"
        }

    def load_split_test(self, split):
        self._load_split(split)

    def prepare_table(self, entry):
        t = Table()
        t.type = "default"
        t.default_question = entry["question"]

        t.props = entry['properties']
        t.txt_info = entry['txt']
        t.html = entry["html"]
        
        return t
if __name__ == '__main__':
    MultiHiertt().load_split_test("train")


import torch
import torch.nn as nn
from torch.utils.data import Dataset, DataLoader
import numpy as np
from transformers import BartTokenizer, BartForConditionalGeneration
from transformers import RobertaTokenizer, RobertaModel, BertTokenizer, BertModel, AutoTokenizer, AutoModel
import json
from tqdm import tqdm
import random
from pathlib import Path
import logging
import os, sys, math
import torch.nn.functional as F
from transformers import get_linear_schedule_with_warmup, AdamW
import pickle
import argparse


##  添加了AdamW和linear warm up
def create_logger(name, silent=False, to_disk=True, log_file=None):
    """Logger wrapper
    """
    # setup logger
    log = logging.getLogger(name)
    log.setLevel(logging.DEBUG)
    log.propagate = False
    formatter = logging.Formatter(fmt='%(asctime)s %(message)s', datefmt='%m/%d/%Y %I:%M:%S')
    if not silent:
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logging.INFO)
        ch.setFormatter(formatter)
        log.addHandler(ch)
    if to_disk:
        log_file = log_file if log_file is not None else strftime("%Y-%m-%d-%H-%M-%S.log", gmtime())
        fh = logging.FileHandler(log_file)
        fh.setLevel(logging.DEBUG)
        fh.setFormatter(formatter)
        log.addHandler(fh)
    return log

def load_data(path):
    with open(path, 'r') as f:
        data = json.load(f)
    return data

def gelu(x):
    return x * 0.5 * (1.0 + torch.erf(x / math.sqrt(2.0)))
class FFNLayer(nn.Module):
    def __init__(self, input_dim, intermediate_dim, output_dim, dropout, layer_norm=True):
        super(FFNLayer, self).__init__()
        self.fc1 = nn.Linear(input_dim, intermediate_dim)
        if layer_norm:
            self.ln = nn.LayerNorm(intermediate_dim)
        else:
            self.ln = None
        self.dropout_func = nn.Dropout(dropout)
        self.fc2 = nn.Linear(intermediate_dim, output_dim)

    def forward(self, input):

        inter = self.fc1(self.dropout_func(input))
        inter_act = gelu(inter)
        if self.ln:
            inter_act = self.ln(inter_act)
        return self.fc2(inter_act)    

class RetrieveModel(nn.Module):
    def __init__(self, bert_model):
        super(RetrieveModel, self).__init__()
        self.bert_model = bert_model
        self.hidden_size = self.bert_model.embeddings.word_embeddings.embedding_dim
        self.projection = FFNLayer(self.hidden_size, self.hidden_size, 1, 0.2)

    def forward(self, data):
        inputs = {"input_ids": data['input_ids'], "attention_mask": data['input_mask']}
        cls_output = self.bert_model(**inputs)[0][:,0,:]
        logits = self.projection(cls_output)
        bs = data['labels'].size(0)
        device = cls_output.device
        probs = logits.squeeze(-1).unsqueeze(0)
        probs = torch.softmax(probs, -1)
        return probs



class TypeDataset(Dataset):
    def __init__(self, tokenizer, data, is_train, data_use, rerank_link, is_test=0) -> None:
        super(TypeDataset, self).__init__()
        self.tokenizer = tokenizer
        total_data = []
        self.is_train = is_train
        self.is_test = is_test
        self.data_use = data_use  
        self.rerank_link = rerank_link   
        if data_use == 0:    
            total_data = data
        elif data_use == 1:   
            for item in data:
                if sum(item['labels']) != 0:
                    total_data.append(item)
        elif data_use == 2:   
            for item in data:
                if sum(item['labels']) == 1:
                   total_data.append(item)
        self.data = []
        # import pdb; pdb.set_trace()
        for data in tqdm(total_data):
            path = '/home/lfy/UMQM/Data/HybridQA/WikiTables-WithLinks'
            table_id = data['table_id']
            with open('{}/tables_tok/{}.json'.format(path, table_id), 'r') as f:
                table = json.load(f)  
            with open('{}/request_tok/{}.json'.format(path, table_id), 'r') as f:
                requested_document = json.load(f)
            dot_token = self.tokenizer.additional_special_tokens[0]
            dot_token_id = self.tokenizer.convert_tokens_to_ids(dot_token)
            question_ids = self.tokenizer.encode(data['question'])
            headers = [_[0] for _ in table['header']]
            row_tmp = '{} is {} {}'
            row_links = []
            input_ids = []
            for i, row in enumerate(table['data']):
                row_ids = []
                links = []
                for j, cell in enumerate(row):
                    if cell[0] != '':
                        cell_desc = row_tmp.format(headers[j], cell[0], dot_token)
                        cell_toks = self.tokenizer.tokenize(cell_desc)
                        cell_ids = self.tokenizer.convert_tokens_to_ids(cell_toks)
                        row_ids += cell_ids
                    if cell[1] != []:
                        links += cell[1]
                row_links.append(links.copy())
                if rerank_link:
                    links = self.generate_new_links(links, data)
                for link in links:
                    passage_toks = self.tokenizer.tokenize(requested_document[link])
                    passage_ids = self.tokenizer.convert_tokens_to_ids(passage_toks)
                    row_ids += passage_ids + [dot_token_id]
                row_ids = question_ids + row_ids + [self.tokenizer.sep_token_id]
                input_ids.append(row_ids)
            data['input_ids'] = input_ids
            data['row_links'] = row_links
            self.data.append(data)
    
    def generate_new_links(self, links, data):
        if self.is_train:
            answer_link = [item[2] for item in data['answer-node']]
            new_links, other_links = [], []
            for link in links:
                if link in answer_link:
                    new_links.append(link)
                else:
                    other_links.append(link)
            new_links += other_links
            return new_links
        else:
            aa = 1
            links_rank = data['links_rank']
            total_links = data['links']
            row_link_id = [total_links.index(item) for item in links]
            row_link_id_rank = [links_rank.index(item) for item in row_link_id]
            final_rank = np.argsort(row_link_id_rank).tolist()
            new_links = []
            for item in final_rank:
                new_links.append(links[item])
            return new_links
    def __len__(self):
        return len(self.data)
    def __getitem__(self, index):
        data = self.data[index]
        if not self.is_test:
            return data['input_ids'], data['labels'], data
        else:
            return data['input_ids'], [0]*len(data['input_ids']), data
            
                    
def collate(data, tokenizer, bert_max_length, is_test=0):
    pad_id = tokenizer.convert_tokens_to_ids(tokenizer.pad_token)
    row_num = [len(item[1]) for item in data]
    max_row_num = max(row_num)
    bs = len(data)
    max_input_length = 0
    for i in range(bs):
        rows_i = data[i][0]
        max_row_length_i = max([len(i) for i in rows_i])
        if max_row_length_i > bert_max_length:
            max_input_length = bert_max_length
            break
        if max_row_length_i > max_input_length:
            max_input_length = max_row_length_i
  
    input_ids = []
    row_mask = torch.zeros(bs, max_row_num)
    labels = torch.zeros(bs, max_row_num)
    metadata = []
    for i in range(bs):
        row_mask[i][:row_num[i]] = 1
        labels[i][:len(data[i][1])] = torch.tensor(data[i][1])
        input_data_i = []
        for item in data[i][0]:
            if len(item) > bert_max_length:
                item = item[:bert_max_length]
            else:
                item = item + (max_input_length - len(item)) * [pad_id]
            input_data_i.append(item)
        input_ids.extend(input_data_i)
        metadata.append(data[i][2])       
    input_ids = torch.tensor(input_ids)
    input_mask = torch.where(input_ids==tokenizer.pad_token_id, 0, 1)
    if not is_test:
        return {"input_ids": input_ids.cuda(), "input_mask":input_mask.cuda(), "labels":labels.cuda(),\
            "row_mask": row_mask.cuda(), "metadata":metadata, "max_row_num": max_row_num}
    else:
        return {"input_ids": input_ids.cuda(), "input_mask":input_mask.cuda(),"labels":labels.cuda(),\
            "row_mask": row_mask.cuda(), "metadata":metadata, "max_row_num": max_row_num}


def train(epoch, tokenizer, model, loader, optimizer, scheduler, logger, is_firststage):
    model.train()
    averge_step = len(loader) // 12
    loss_sum, step = 0, 0
    for i, data in enumerate(tqdm(loader)):
        probs = model(data)
        if not is_firststage:
            if sum(data['labels'][0]).cpu().item()!=1:
                data['labels'] = F.softmax(probs + (1 - data['labels'].float()) * -1e20, dim=-1)
        loss_func = nn.BCEWithLogitsLoss(reduction='sum')
        loss = loss_func(probs, data['labels'])
        optimizer.zero_grad()
        loss.backward()
        torch.nn.utils.clip_grad_norm_(model.parameters(), 0.1)
        optimizer.step()
        scheduler.step()
        loss_sum += loss
        step += 1
        if i % averge_step == 0:
            logger.info("Training Loss [{0:.5f}]".format(loss_sum/step))
            loss_sum, step = 0, 0

def eval(model, loader, logger):
    model.eval()
    total, acc = 0, 0
    with torch.no_grad():
        for i, data in enumerate(tqdm(loader)):
            probs = model(data)
            predicts = torch.argmax(F.softmax(probs,dim=1), dim=1).cpu().tolist()
            gold_row = [np.where(item==1)[0].tolist() for item in data['labels'].cpu().numpy()]
            score_rank = torch.argsort(-probs, dim=1).cpu().tolist()[0]
            scores, row_rank = torch.sort(probs, dim=1, descending=True)[0][0].cpu().numpy(), torch.sort(probs, dim=1, descending=True)[1][0].cpu().numpy() 
            
            data['metadata'][0]['row_links']
            question = data['metadata'][0]['question']
            row_links = data['metadata'][0]['row_links']
            links = data['metadata'][0]['links']
            links_rank = data['metadata'][0]['links_rank']
            link_labels = data['metadata'][0]['link_labels']
            labels = data['metadata'][0]['labels']

            for i in range(len(predicts)):
                total += 1
                if predicts[i] in gold_row[i]:
                    acc += 1
    print(f"Total: {total}")
    return acc / total

def eval_file(model, loader, logger):
    model.eval()
    total, acc = 0, 0
    outputs = []
    with torch.no_grad():
        for i, data in enumerate(tqdm(loader)):
            probs = model(data)
            predicts = torch.argmax(F.softmax(probs,dim=1), dim=1).cpu().tolist()
            predcits_logits = probs.cpu().detach().tolist()
            gold_row = [np.where(item==1)[0].tolist() for item in data['labels'].cpu().numpy()]
            metadatas = data['metadata']
            for j, metadata in enumerate(metadatas):
                metadata['row_pre'] = predicts[j]
                metadata['row_gold'] = gold_row[j]
                metadata['row_pre_logit'] = predcits_logits[j]
                outputs.append(metadata)
            for i in range(len(predicts)):
                total += 1
                if predicts[i] in gold_row[i]:
                    acc += 1

    return acc / total, outputs

def test_file(model, loader, logger):
    model.eval()
    total, acc = 0, 0
    outputs = []
    with torch.no_grad():
        for i, data in enumerate(tqdm(loader)):
            probs = model(data)
            predicts = torch.argmax(F.softmax(probs,dim=1), dim=1).cpu().tolist()
            predcits_logits = probs.cpu().detach().tolist()
            metadatas = data['metadata']
            for j, metadata in enumerate(metadatas):
                metadata['row_pre'] = predicts[j]
                metadata['row_pre_logit'] = predcits_logits[j]
                metadata.pop('input_ids')
                outputs.append(metadata)
    return outputs

def read_config(path):
    with open(path, 'r') as file:
        config = json.load(file)
    return config

def main():
    device = torch.device("cuda")
    parser = argparse.ArgumentParser()
    parser.add_argument('--ptm_type', type=str, default='bert-base', help='Pre-trained model to use')
    parser.add_argument('--train_data_path', type=str, default='./Data/HybridQA/train.p.json', help='Path to training data')
    parser.add_argument('--dev_data_path', type=str, default='./Data/HybridQA/dev.p.json', help='Path to development data')
    parser.add_argument('--predict_save_path', type=str, default='./Data/HybridQA/dev.row.json', help='Path to save predictions')
    parser.add_argument('--batch_size', type=int, default=1, help='Batch size')
    parser.add_argument('--epoch_nums', type=int, default=5, help='Number of epochs')
    parser.add_argument('--learning_rate', type=float, default=7e-6, help='Learning rate')
    parser.add_argument('--adam_epsilon', type=float, default=1e-8, help='Epsilon for Adam optimizer')
    parser.add_argument('--warmup_steps', type=int, default=0, help='Number of warmup steps for learning rate scheduler')
    parser.add_argument('--is_train', type=int, default=1, help='Whether to train the model')
    parser.add_argument('--is_test', type=int, default=0, help='Whether to test the model')
    parser.add_argument('--is_generate', type=int, default=0, help='Whether to generate predictions')
    parser.add_argument('--is_firststage', type=int, default=1, help='Whether to perform first-stage retrieval')
    parser.add_argument('--rerank_link', type=int, default=1, help='Whether to rerank links')
    parser.add_argument('--seed', type=int, default=2001, help='Random seed')
    parser.add_argument('--output_dir', type=str, default='./retrieve1', help='Output directory')
    parser.add_argument('--load_dir', type=str, default='./retrieve1', help='Directory to load model from')
    parser.add_argument('--bert_max_length', type=int, default=512, help='Maximum length of input sequence for BERT')
    parser.add_argument('--config_file', type=str, default='config_retrieve1.json')

    args = parser.parse_args()
    
    training_config = read_config(args.config_file)

    ptm_type = args.ptm_type
    train_data_path = args.train_data_path
    dev_data_path = args.dev_data_path
    predict_save_path = args.predict_save_path
    warmup_steps = args.warmup_steps
    is_train = args.is_train
    is_test = args.is_test
    is_generate = args.is_generate
    is_firststage = args.is_firststage
    rerank_link = args.rerank_link
    seed = args.seed
    output_dir = args.output_dir
    load_dir = args.load_dir
    bert_max_length = args.bert_max_length
    
    batch_size = training_config['batch_size']
    epoch_nums = training_config['epoch_nums']
    bert_max_length = training_config['bert_max_length']
    learning_rate = training_config['learning_rate']
    adam_epsilon = training_config['adam_epsilon']



    log_file = 'log.txt'
    ckpt_file = 'ckpt.pt'
    load_ckpt_file = 'ckpt.pt'
    n_gpu = torch.cuda.device_count()
    notice = f'is_train={is_train}, is_test={is_test}, is_firststage={is_firststage}, lr={learning_rate}, epoch_num={epoch_nums}'

    Path(output_dir).mkdir(parents=True, exist_ok=True)
    Path(load_dir).mkdir(parents=True, exist_ok=True)
    logger = create_logger("Training", log_file=os.path.join(output_dir, log_file))

    random.seed(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    if n_gpu > 0:
        torch.cuda.manual_seed_all(seed)
        

        
    logger.info(f"{notice}")
    logger.info(f"load_dir: {load_dir}   output_dir: {output_dir}")
    logger.info(f"loading data......from {train_data_path} and {dev_data_path}")
    train_data, dev_data = load_data(train_data_path), load_data(dev_data_path)
    logger.info(f"train data: {len(train_data)}, dev data: {len(dev_data)}")
    
    if ptm_type == 'bert-large':
        ptm_path = './PTM/bert-large-uncased'
        logger.info(f"loading PTM model......from {ptm_path}")
        tokenizer = BertTokenizer.from_pretrained(ptm_path)
        bert_model = BertModel.from_pretrained(ptm_path)
        special_tokens_dict = {'additional_special_tokens': ['[DOT]']}
        tokenizer.add_special_tokens(special_tokens_dict)
        bert_model.resize_token_embeddings(len(tokenizer))
    elif ptm_type == 'bert-base':
        ptm_path = './PTM/bert-base-uncased'
        logger.info(f"loading PTM model......from {ptm_path}")
        tokenizer = BertTokenizer.from_pretrained(ptm_path)
        bert_model = BertModel.from_pretrained(ptm_path)
        special_tokens_dict = {'additional_special_tokens': ['[DOT]']}
        tokenizer.add_special_tokens(special_tokens_dict)
        bert_model.resize_token_embeddings(len(tokenizer))
    elif ptm_type == 'deberta':
        ptm_path = './PTM/deberta-base'
        logger.info(f"loading PTM model......from {ptm_path}")
        tokenizer = AutoTokenizer.from_pretrained(ptm_path)
        bert_model = AutoModel.from_pretrained(ptm_path)
        special_tokens_dict = {'additional_special_tokens': ['[DOT]']}
        tokenizer.add_special_tokens(special_tokens_dict)
        bert_model.resize_token_embeddings(len(tokenizer))

    if is_firststage:
        if is_train:
            train_dataset = TypeDataset(tokenizer, train_data, is_train=1, data_use=2, rerank_link=rerank_link)
        dev_dataset = TypeDataset(tokenizer, dev_data, is_train=0, data_use=2, rerank_link=rerank_link, is_test=is_test)
    else:
        if is_train:
            train_dataset = TypeDataset(tokenizer, train_data, 1, 1, rerank_link)
        dev_dataset = TypeDataset(tokenizer, dev_data, 0, 0, rerank_link, is_test)


    if is_train:
        logger.info(f"train dataset: {len(train_dataset)}")
    logger.info(f"dev dataset: {len(dev_dataset)}")

    if is_train:
        train_loader = DataLoader(train_dataset, batch_size=batch_size, collate_fn=lambda x: collate(x, tokenizer, bert_max_length, is_test))
    dev_loader = DataLoader(dev_dataset, batch_size=batch_size, collate_fn=lambda x: collate(x, tokenizer, bert_max_length, is_test))
    
    model = RetrieveModel(bert_model)
    model.to(device)

    if not is_firststage and is_train:
        model_load_path = os.path.join(load_dir, load_ckpt_file)
        logger.info(f"loading trained parameters from {model_load_path}")
        model.load_state_dict(torch.load(model_load_path))

    best_acc = 0
    if is_train:
        if n_gpu > 1:
            model = torch.nn.DataParallel(model)
        optimizer = AdamW(model.parameters(), lr=learning_rate, eps=adam_epsilon)
        t_total = len(train_dataset) * epoch_nums
        scheduler = get_linear_schedule_with_warmup(
            optimizer, num_warmup_steps=warmup_steps * t_total, num_training_steps=t_total
        )

        for epoch in range(epoch_nums):
            logger.info(f"Training epoch: {epoch}")
            train(epoch, tokenizer, model, train_loader, optimizer, scheduler, logger, is_firststage)
            logger.info("start eval....")
            acc = eval(model, dev_loader, logger)
            logger.info(f"acc... {acc}")
            if acc > best_acc:
                best_acc = acc
                model_to_save = model.module if hasattr(model, "module") else model
                model_save_path = os.path.join(output_dir, ckpt_file)
                logger.info(f"saving model...to {model_save_path}")
                torch.save(model_to_save.state_dict(), model_save_path)
    else:
        model_load_path = os.path.join(load_dir, load_ckpt_file)
        logger.info(f"loading trained parameters from {model_load_path}")
        model.load_state_dict(torch.load(model_load_path))
        logger.info("start eval....")
        if is_generate:
            if not is_test:
                acc, outputs = eval_file(model, dev_loader, logger)
                print(f"acc: {acc}")
            else:
                outputs = test_file(model, dev_loader, logger)
            with open(predict_save_path, 'w') as f:
                json.dump(outputs, f, indent=2)
            logger.info(f"saving to {predict_save_path}")
            
        else:
            acc = eval(model, dev_loader, logger)
            print(f"acc: {acc}")
    

if __name__ == '__main__':
    main()


from pathlib import Path
from setuptools import find_packages, setup


project_root = Path(__file__).parent
install_requires = [
    "Flask>=2.2.2",
    "datasets>=2.9.0",
    "requests",
    "lxml",
    "tinyhtml",
    "xlsxwriter",
    "coloredlogs",
]

setup(
    name="ttqakit",
    version="0.1.0",
    python_requires=">=3.8",
    description="TTQAKit: A toolkit for Text-Table Hybrid Question Answering.",
    author="Fangyu Lei",
    author_email="leifangyu@qq.com",
    long_description=(project_root / "README.md").read_text(encoding="utf-8"),
    long_description_content_type="text/markdown",
    url="https://github.com/lfy79001/TextTableQAKit",
    license="Apache-2.0 License",
    packages=find_packages(exclude=["test", "test.*"]),
    # package_dir={"": "src"},
    # package_data={
    #     "tabgenie": ["static/css/*", "static/img/*", "static/js/*", "templates/*"],
    # },
    # data_files=[("tabgenie", ["tabgenie/config.yml"])],
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "tabgenie=tabgenie.cli:run",
        ],
        "flask.commands": ["export=tabgenie.cli:export", "sheet=tabgenie.cli:sheet", "info=tabgenie.cli:info"],
    },
    install_requires=install_requires,
    extras_require={
        "dev": [
            "wheel",
            "black",
        ],
        "deploy": [
            "gunicorn",
        ],
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Intended Audience :: Science/Research",
        "Operating System :: POSIX :: Linux",
        "License :: OSI Approved :: Apache Software License",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Typing :: Typed",
    ],
)


def change_output2(output, pre_order_tokens, question_id):
    try:
        if len(output) % 2 == 0:
            if output[0] == output[1]:
                output.pop(0)
                pre_order_tokens.pop(0)
            elif output[1] == output[2]:
                output.pop(1)
                pre_order_tokens.pop(1)
            elif output[2] == output[3]:
                output.pop(2)
                pre_order_tokens.pop(2)
            elif output[4] == output[5]:
                output.pop(4)
                pre_order_tokens.pop(4)
            elif output[7] == output[8]:
                output.pop(7)
                pre_order_tokens.pop(7)
    except:
        import pdb; pdb.set_trace()    
    return output, pre_order_tokens


from transformers import AutoModel
from transformers import AutoTokenizer
import torch
import torch.nn as nn
import math
from torch.nn import functional as F

def gelu(x):
    return x * 0.5 * (1.0 + torch.erf(x / math.sqrt(2.0)))

class FFNLayer(nn.Module):
    def __init__(self, input_dim, intermediate_dim, output_dim, dropout, layer_norm=True):
        super(FFNLayer, self).__init__()
        self.fc1 = nn.Linear(input_dim, intermediate_dim)
        if layer_norm:
            self.ln = nn.LayerNorm(intermediate_dim)
        else:
            self.ln = None
        self.dropout_func = nn.Dropout(dropout)
        self.fc2 = nn.Linear(intermediate_dim, output_dim)

    def forward(self, input):

        inter = self.fc1(self.dropout_func(input))
        # inter_act = gelu(inter)
        inter_act = F.gelu(inter) # ?
        if self.ln:
            inter_act = self.ln(inter_act)
        return self.fc2(inter_act)


class ClassifierModel(nn.Module):
    """
        classify questions with table, image and passages into four types (image, table, text, compose)
    """
    def __init__(self, bert_model = 'microsoft/deberta-v3-large', num_classes = 4, dropout = 0.2):
        super(ClassifierModel, self).__init__()
        if type(bert_model) == str:
            bert_model = AutoModel.from_pretrained(bert_model)
        self.bert_model = bert_model
        self.hidden_size = self.bert_model.embeddings.word_embeddings.embedding_dim
        self.projection = FFNLayer(self.hidden_size, self.hidden_size, num_classes, dropout=dropout)

    def forward(self, data):
        # print(data)
        inputs = {"input_ids": data['input_ids'], "attention_mask": data['input_mask']}
        cls_output = self.bert_model(**inputs)[0][:,0,:]
        logits = self.projection(cls_output)
        # bs = data['labels'].size(0)
        # device = cls_output.device
        # probs = logits.squeeze(-1).unsqueeze(0)
        probs = logits.squeeze(-1)
        # probs = torch.softmax(probs, -1)
        return probs

if __name__ == '__main__':
    bert_model = AutoModel.from_pretrained("microsoft/deberta-v3-large")
    tokenizer = AutoTokenizer.from_pretrained("microsoft/deberta-v3-large")
    model = ClassifierModel(bert_model)
